#include "Flags.h"




FLAGS::FLAGS(Image<float>& f,float low): Image<int>(f.getWidth(),f.getHeight())
//Initialize this by thresholding 'f' with value 'low'.
//Additionally, 'f' is modified to reflect the thresholding. Thresholding classifies
//pixels in three groups:
//    -outside:    pixels where f<low, if low>=0, or f>-low, if low<0
//    -inside:     pixels which are not outside
//    -narrowband: pixels on the inside-outside border 
//The values are set as follows:
//
//   -outside:     this = ALIVE,       f = 0
//   -inside:      this = FAR_AWAY,    f = infinity
//   -narrowband:  this = NARROW_BAND, f = 1
{									
   nband = ninside = 0;

   for(int j=0;j<getHeight();++j)	
   for(int i=0;i<getWidth();++i)	
   {
		float& v = f(j,i);
		int&   c = (*this)(j,i);

		if ((low>=0 && v<low) || (low<0 && v>-low))
		{ v = 0; c = ALIVE; }
		else
		{ v = MYINFINITY; c = FAR_AWAY; ninside++; }
   }

   for(int j=0;j<getHeight();j++)
      for(int i=0;i<getWidth();i++)
      if (faraway(i,j))
		if (alive(i-1,j) || alive(i+1,j) || alive(i,j-1) || alive(i,j+1))
		{ (*this)(j,i) = NARROW_BAND; f(j,i) = 1; nband++; }

   ninside -= nband;
}


