#include "Image.h"



//-----  Specializations  --------------------------------


template<> Image<RGBPixel>* Image<RGBPixel>::readPPM(const char* filename)	
{														//Read PPM file into color image. Since we have 3 channels, we 
   FILE* fp = fopen(filename,"rb"); if (!fp) return 0;	//can now store all colors separately (compare with the non-specialized method).
   
   const int SIZE = 3*1024;
   char buf[SIZE]; int dimX,dimY,range;
   fscanf(fp,"%*s");									//skip "P6" header

   for(;;)
   {
     fscanf(fp,"%s",buf);								//get dimX or #comment
     if (buf[0]=='#') fgets(buf,SIZE,fp); 
        else { dimX = atoi(buf); break; }
   }
   for(;;)
   {
     fscanf(fp,"%s",buf);								//get dimY or #comment
     if (buf[0]=='#') fgets(buf,SIZE,fp); 
        else { dimY = atoi(buf); break; }
   }
   for(;;)
   {
     fscanf(fp,"%s",buf);								//get range or #comment
     if (buf[0]=='#') fgets(buf,SIZE,fp); 
        else { range = atoi(buf); break; }
   }
   
   Image<RGBPixel>* f = new Image<RGBPixel>(dimX,dimY);
   int bb = SIZE; fgets(buf,SIZE,fp);
  
   for(int j=0;j<dimY;++j)
   {
	  for(int i=0;i<dimX;++i)							//read the binary data into the image
	  {													//do not modify the range of the read pixels
		if (bb==SIZE) { fread(buf,SIZE,1,fp); bb=0; }
		
		float r = (unsigned char)buf[bb++];
		float g = (unsigned char)buf[bb++];
		float b = (unsigned char)buf[bb++];
		
		(*f)(j,i) = RGBPixel(r,g,b);					//store a new color pixel in the image
	  }	
   }

   fclose(fp);  				
   return f;
}




template<> void Image<RGBPixel>::writePGM(const char* fname) const
{
   FILE* fp = fopen(fname,"wb");
   if (!fp) return;

   const int SIZE = 1024;
   unsigned char buf[SIZE];
   int bb=0;

   fprintf(fp,"P5 %d %d 255\n",getWidth(),getHeight());
   
   for(int j=0;j<getHeight();++j)
   {
	  for(int i=0;i<getWidth();++i)						//
	  {													//
	     const RGBPixel& v = (*this)(j,i);
         buf[bb++] = (unsigned char)(int)((v.r+v.g+v.b)/3);
		 if (bb==SIZE)
         {  fwrite(buf,sizeof(unsigned char),SIZE,fp); bb = 0; }
	  }	 
   }
   if (bb) fwrite(buf,sizeof(unsigned char),bb,fp);

   fclose(fp);
}	


template<> void Image<RGBPixel>::writePPM(const char* fname) const
{
   FILE* fp = fopen(fname,"wb");
   if (!fp) return;

   const int SIZE = 3*1024;
   unsigned char buf[SIZE];
   int bb=0;

   fprintf(fp,"P6 %d %d 255\n",getWidth(),getHeight());
   
   for(int j=0;j<getHeight();++j)
   {
	  for(int i=0;i<getWidth();++i)						//
	  {													//
	     const RGBPixel& v = (*this)(j,i);
         buf[bb++] = (unsigned char)(int)(v.r);
         buf[bb++] = (unsigned char)(int)(v.g);
         buf[bb++] = (unsigned char)(int)(v.b);

		 if (bb==SIZE)
         {  fwrite(buf,sizeof(unsigned char),SIZE,fp); bb = 0; }
	  }	 
   }
   if (bb) fwrite(buf,sizeof(unsigned char),bb,fp);

   fclose(fp);
}	




