#pragma once

#include "Flags.h"
#include <map>



class 	FastMarchingMethod					//Implements the Fast Marching Method on 2D floating-point images
{
public:	

struct Coord								//Coordinate of a pixel in a 2D image
{ 
	int i,j; 
	Coord(int i_,int j_):i(i_),j(j_) {}; 
	Coord() {} 
	int operator==(const Coord& c) const { return i==c.i && j==c.j; } 
};

static const int MAX_ITERATIONS = 10000000;

			FastMarchingMethod(Image<float>*,FLAGS*);
											//Initializes the FMM engine to compute, in the 1st argument, 
											//the distance transform of the boundary encoded by the 2nd argument
															
virtual	   ~FastMarchingMethod();
virtual int execute(float max_distance=FLAGS::MYINFINITY,int max_iterations=MAX_ITERATIONS);		
											//Compute the DT. Stop when either the maximum number of iterations was
											//exceeded, or (locally) when a pixel's distance exceeds the 
											//given maximum distance. Returns number of iterations done.

void		setVerbose(bool b)				//Tell if should display info during execution or not
			{ verbose = b; }

protected:

virtual void add_to_narrowband(int,int,int,int);						
											//Called by execute() whenever a FAR_AWAY
											//point is first added to the narrowband.	
											//First 2 arguments: point to be added. 
											//Last 2 arguments: active-point-neighbor causing addition of above.
											//Subclasses could enhance this if they want
											//to do extra work when adding a point to
											//the narrowband

typedef Image<std::multimap<float,Coord>::iterator> POINTERS;
											//Maps distance values to distance-map locations where they are realized

void    	 tag_nbs(int,int,int,int,Coord*,int&);
virtual int  diffuse();
virtual void solve(int,int,float,float,float&);		

std::multimap<float,Coord> map;				//Narrowband points sorted in ascending distance order
Image<float>*			   f;				//Distance field. Holds the result (distance transform)
FLAGS*					   flags;			//Pixel classification into ALIVE,NARROWBAND, and FAR_AWAY
POINTERS				   ptrs;			//Map from a distance value to positions in 'map' where this is realized
int						   N;				//Max #iterations allowed
int						   negd;			//Number of failures in solve2()
int						   nextr;			//Number of extremum points detected in diffuse()
float					   maxf;			//Threshold to stop evolution (see execute()).
bool					   verbose;			//If true, displays info during execution
};	
