#pragma once


// FLAGS:	Classification of pixels in a floating-point image, with respect to a threshold value.
//			The class is intended to be used by FastMarchingMethod. It can, however, also be used 
//			for other problems requiring pixel classification.
//			Pixels can be of four kinds:
//
//			ALIVE:		    pixels are outside the isoline determined by the threshold value.
//			NARROW_BAND:	pixels are on the isoline
//			FAR_AWAY:	    pixels are inside the isoline
//			EXTREMUM:	    special case for ALIVE pixels. Extremum values are ALIVE values which
//							are also detected as extremum points.
//	
// FLAGS objects can be constructed from floating-point fields by thresholding, via the constructor FLAGS(f,low):
//
//			If low >= 0:	outside pixels are those pixels which have a f value < low
//			If low < 0:     outside pixels are those pixels which have a f value > -low
//
	

#include "Image.h"


class FLAGS : public Image<int>
{
public:

	static const float MYINFINITY = 1.0e7f;

	enum FLAG_TYPE { NARROW_BAND,ALIVE,FAR_AWAY,EXTREMUM };

		 FLAGS(Image<float>& f,float low);		 

	bool alive(int i,int j) const 		{ return (*this)(j,i)==ALIVE;		}
	bool narrowband(int i,int j) const	{ return (*this)(j,i)==NARROW_BAND; }
	bool faraway(int i,int j) const		{ return (*this)(j,i)==FAR_AWAY;	}
	bool extremum(int i,int j) const    { return (*this)(j,i)==EXTREMUM;	}
	int  initialContourLength() const 	{ return nband;	  }
	int  initialObjectSize() const      { return ninside; } 

private:

int		 nband;
int      ninside;
};

